var rrm = new RRM(); 

function RRM() {
	var rrm = this;

    this.imageBounds = [[48,123], [22,153]];
	this.map = null;
    this.enabled = false;
    this.isPaused = false;

    this.layers = [];
    this.i = null;
    this.firstLoad = false;

    this.lt = null;
    this.lp = null;
    this.multi = 3;
    this.refreshImage = true;
    this.delayActive = false;

    this.marker = null;

    this.init = function() {
        rrm.map = L.map('radar-map-wrapper', {
			zoomControl: false,
			minZoom: 4,
			maxZoom: 10,
            fadeAnimation: false,
			maxBounds: [[48,123], [22,153]]
		}).setView([userLat, userLong], 7);

		rrm.map.attributionControl.setPrefix("© OpenStreetMap contributors");
		L.tileLayer.grayscale('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png').addTo(rrm.map);

        rrm.enabled = true;
		rainRadar.resetMapViewFlag = false;
        rrm.startImageRotator();
    };

    this.startImageRotator = function() {
        if(rrm.lp === null && rrm.isPaused === false) {

            if(rrm.i == null) {
                rrm.i = 0;
            }

			$("#slide").attr("max", radarData.length - 1);

            rrm.lp = setInterval(function() {
                if(rrm.refreshImage === true) {
                    rrm.refreshImage = false;

                    $("#slide").val(rrm.i);
                    var labelDate = layers.getTimestampLabel(rrm.i);
                    $(".radar-label .right").html(labelDate);      

                    //set imagelayer if doesnt exist yet
                    if(typeof rrm.layers[rrm.i] === 'undefined') {
                        rrm.layers[rrm.i] = L.tileLayer(radarData[rrm.i], {
                            opacity: 1
                        });
                        
                        rrm.layers[rrm.i].on('load', function(ev) {
                            $.each(rrm.layers, function(index, layer) {
                                if(index != rrm.i && rrm.map.hasLayer(layer)) {
                                    layer.setOpacity(0);
                                }
                            });
                            rrm.mapAnimationDelay();
                        });

                        rrm.layers[rrm.i].on('error', function(ev) {
                            rrm.mapAnimationDelay();
                        });

                        rrm.layers[rrm.i].addTo(rrm.map);
                    }
                    else {
                        $.each(rrm.layers, function(index, layer) {
                            if(index != rrm.i && rrm.map.hasLayer(layer)) {
                                layer.setOpacity(0);
                            }
                        });
                        rrm.layers[rrm.i].setOpacity(1);
                        rrm.mapAnimationDelay();
                    }
                }
            }, 100);
        }
    };

    this.mapAnimationDelay = function() {
        if(rrm.delayActive === true) return;
        rrm.delayActive = true;

        var tMulti = 1;
        if(rrm.i >= radarData.length - 1) {
            tMulti = 2;
        }

        rrm.lt = setTimeout(function() {
            rrm.i = (rrm.i == (radarData.length - 1)) ? 0 : rrm.i + 1;
            rrm.refreshImage = true;
            rrm.delayActive = false;
        }, 1500 * tMulti);
    }

    this.stopImageRotator = function() {
        rrm.delayActive = false;

		if(rrm.lp !== null) {
        	clearInterval(rrm.lp);
            rrm.lp = null;
        }
        if(rrm.lt !== null) {
			clearTimeout(rrm.lt);
			rrm.lt = null;
		}
    };

    this.resetMapView = function() {
		var zoomLevel = 7;
		rrm.map.setView([userLat, userLong], zoomLevel);
	};

    this.showSelectedImage = function(sI) {
		$(".radar-loader").show();
		rrm.i = parseInt(sI);
		$(".radar-label .right").html(layers.getTimestampLabel(rrm.i));
		
		$.each(rrm.layers, function(index, layer) {
			if(index != rrm.i && rrm.map.hasLayer(layer)) {
				layer.setOpacity(0);
			}
		});
		
		if(typeof rrm.layers[rrm.i] === 'undefined') {
            rrm.layers[rrm.i] = L.tileLayer(radarData[rrm.i], {
                opacity: 1
            }).addTo(rrm.map);
		}
		else {
			rrm.layers[rrm.i].setOpacity(1);
		}

		$(".radar-loader").hide();
	};
	
	this.clearImageLayers = function () {
		$.each(rrm.layers, function(index, layer) {
			if(rrm.map.hasLayer(layer)) {
				rrm.map.removeLayer(layer);
			}
		});

        rrm.refreshImage = true;
		rrm.layers = [];
		rrm.i = null;
	};

    this.setMarker = function(lat, long) {
		var markerIcon = L.divIcon({
			className: 'css-icon',
			html: '<span id="radar-marker"></span>',
			iconSize: [40, 40]
		});

		if(rrm.marker === null) {
			rrm.marker = L.marker([lat, long], {
				icon: markerIcon
			});
			rrm.marker.addTo(rrm.map);
		}
		else {
			rrm.marker.setLatLng([lat, long]);
		}
	};
    
    this.isEnabled = function () {
		return rrm.enabled;
	};
}

$("#slide").on('touchstart', function(ev) {
	rrm.stopImageRotator();
});

$("#slide").on('touchend', function(ev) {
	if(rrm.isPaused === false) {
		rrm.startImageRotator();
	}
});

$("#slide").on("input", function(ev){
	rrm.isPaused = true;
	$("#play-button").removeClass("fa-pause").addClass("fa-play");
	rrm.showSelectedImage(this.value);
});

$("#play-button").on("touchstart", function(ev) {
	if(rrm.isPaused === false) {
		rrm.isPaused = true;
		rrm.stopImageRotator();
		$("#play-button").removeClass("fa-pause").addClass("fa-play");
	}
	else {
		rrm.isPaused = false;
        rrm.refreshImage = true;
		rrm.startImageRotator();
		$("#play-button").removeClass("fa-play").addClass("fa-pause");
	}
});