var rainRadar = new RainRadar();

function RainRadar() {
    var rainRadar = this;

    this.currentStatus = null;
	this.imageRotator = null;
	this.outlookScroll = null;
    this.resizer = new Resizer();
	this.resetMapViewFlag = false;

	this.locationPermissionAlert = false;
	this.connectionFailedAlert = false;

    this.boolMapLoaded = false;
	this.httpRequestTimeout = 10000;

	this.locationStateEnum = {
		USER_LOCATION: 0,
		USER_FALLBACK_LOCATION: 1,
		DEFAULT_FALLBACK_LOCATION: 2,
		LOCATION_ERROR: 3
	};

	this.locationState = 0;

    this.getUserLocation = function() {
		localforage.getItem(config.storePrefix + "staticLocation").then(function(staticLocation) {
			if(staticLocation != "true") {
				rainRadar.setLocationState(rainRadar.locationStateEnum.USER_LOCATION);
				navigator.geolocation.getCurrentPosition(function(position) {
					userLat = position.coords.latitude;
					userLong = position.coords.longitude;
					rainRadar.getRainRadar();
				},
				function(err) {
					rainRadar.showLocationError(err);
					rainRadar.mapViewDidFail();
					return;
				}, {timeout: 6000});
			}
			else {
				localforage.getItem(config.storePrefix + "lastLocation").then(function(lastLocation) {
					if(lastLocation !== null) {
						if((userLat === null && userLong === null) || rainRadar.isLocationState(rainRadar.locationStateEnum.LOCATION_ERROR)) {
							lastLocation = JSON.parse(lastLocation);
							userLat = lastLocation.lat;
							userLong = lastLocation.long;
						}
					}
					rainRadar.getRainRadar();
				});
			}
		});
	};

	this.setLocationState = function(state) {
		rainRadar.locationState = state;
	};

	this.isLocationState = function(state) {
		return rainRadar.locationState === state;
	};
	
    this.showLocationError = function(error) {
	if(error !== null && error.code == error.PERMISSION_DENIED) {
			if(rainRadar.locationPermissionAlert === false) {
				navigator.notification.alert("前回表示した地域を再表示できます。", null, "位置情報サービスが無効になりました", "Ok");
				rainRadar.locationPermissionAlert = true;
			}
        }
    };

    this.getRainRadar = function(requestWeather) {
		requestWeather = (typeof requestWeather !== 'undefined') ? requestWeather : true;
		mapLayers = layers.buildLayers(radarType);

		if(requestWeather) rainRadar.getWeather();
		rainRadar.setFallbackLocation();
		rainRadar.setIntervalRefreshData();
		rainRadar.setMapView(mapLayers);
    };

	this.getWeather = function() {
		$.ajax(config.apiPath + "twc_forecast_v1.php", {
            type: 'GET',
            dataType: 'json',
            data: {
                "lat": userLat,
				"long": userLong,
				"loc": (locationNameSearch.isStaticLocationName(userLat, userLong) === true) ? "false" : "true"
            },
            timeout: rainRadar.httpRequestTimeout
        }).done(function(results) {
			twc.setData(results);
			if(userLocation === null) userLocation = locationNameSearch.getLocationName(userLat, userLong);
			if(rainRadar.boolMapLoaded) rainRadar.setWeatherView();
		});
	};

    // ==========================
    // Radar Map Functions 
    // ==========================
    this.setMapView = function(mapLayers) {
		if(JSON.stringify(radarData) !== JSON.stringify(mapLayers)) {
			radarData = mapLayers;
			rrm.clearImageLayers();
			if(rrm.isEnabled() === false) {
				$(".splash").fadeOut();
				$(".mapview").fadeIn();
				$(".radar-label").fadeIn();

				if(rainRadar.boolMapLoaded === false && rainRadar.boolWeatherLoaded === true) {
					rainRadar.setWeatherView(weatherData);
				}
				rainRadar.boolMapLoaded = true;
				rainRadar.resize();
				menu.showMenu();

				rrm.init();

				localforage.getItem(config.storePrefix + config.adsManager.isNewUser).then(function(isNewUser) {
					if(isNewUser != "false") {
						localforage.setItem(config.storePrefix + config.adsManager.isNewUser, "false");
						rainRadar.showPopUp();
					}
				});
			}
			else {
				rrm.stopImageRotator();
				rrm.startImageRotator();
			}
		}
		if(rrm.isEnabled()) {
			rrm.setMarker(userLat, userLong);
			if(rainRadar.resetMapViewFlag === true) {
				rrm.resetMapView();
				rainRadar.resetMapViewFlag = false;
			}
		}
		$(".radar-loader").hide();
	};

    this.mapViewLoaded = function() {
        imagesLoaded(".baseelement").on("always", function() {
            $(".splash").fadeOut();
            $(".mapview").fadeIn();
			$(".radar-label").fadeIn();

            if(rainRadar.boolMapLoaded === false && twc.isDataLoaded()) rainRadar.setWeatherView(weatherData);
            rainRadar.boolMapLoaded = true;
            rainRadar.resize();
			menu.showMenu();

			localforage.getItem(config.storePrefix + config.adsManager.isNewUser).then(function(isNewUser) {
				if(isNewUser != "false") {
					localforage.setItem(config.storePrefix + config.adsManager.isNewUser, "false");
					rainRadar.showPopUp();
				}
			});
        });
    };

	this.mapViewDidFail = function() {
		if(rainRadar.isLocationState(rainRadar.locationStateEnum.DEFAULT_FALLBACK_LOCATION)) {
			rainRadar.setLocationState(rainRadar.locationStateEnum.LOCATION_ERROR);
			timeoutUserLocationRetry = setTimeout(function() {
				rainRadar.getUserLocation();
			}, config.timer.userLocationRetry);

			if(rainRadar.connectionFailedAlert === false) {
				navigator.notification.alert("30秒後に再試行します", null, "サーバーに接続出来ません", "Ok");
				rainRadar.connectionFailedAlert = true;
			}
			return;
		}

        rainRadar.getFallbackLocation().then(function(result) {
			timeoutUserLocationPause = setTimeout(function() {
				rainRadar.getUserLocation(false);
			}, config.timer.userLocationPause);
		});
    };

	this.getFallbackLocation = function() {
		return localforage.setItem(config.storePrefix + "staticLocation", "true").then(function(result) {
			return localforage.getItem(config.storePrefix + "lastLocation").then(function(lastLocation) {
				if(rainRadar.isLocationState(rainRadar.locationStateEnum.USER_LOCATION) && lastLocation !== null) {
					rainRadar.setLocationState(rainRadar.locationStateEnum.USER_FALLBACK_LOCATION);
					lastLocation = JSON.parse(lastLocation);
					userLat = lastLocation.lat;
					userLong = lastLocation.long;
					return true;
				}

				rainRadar.setLocationState(rainRadar.locationStateEnum.DEFAULT_FALLBACK_LOCATION);
				userLat = config.userLocationFallback.lat;
				userLong = config.userLocationFallback.long;
				return true;
			});
		});
	};
	
	this.setFallbackLocation = function() {
		rainRadar.setLocationState(rainRadar.locationStateEnum.USER_LOCATION);
		var lastLocation = {
			lat: userLat,
			long: userLong
		};
		localforage.setItem(config.storePrefix + "lastLocation", JSON.stringify(lastLocation));
	};

    this.disableImageRotator = function() {
        if(rainRadar.imageRotator !== null) {
            rainRadar.imageRotator.disable();
            $(".radar-map").fadeOut('fast', function() {
                $(this).remove();
            });
            imagesLoaded(".baseelement").off("always");
            rainRadar.imageRotator = null;
            radarData = null;
        }
    };

    // ==========================
    // Weather Functions 
    // ==========================
    this.setWeatherView = function() {
		var v2obs = twc.v2obs('day_ind');

        if(v2obs === null || v2obs == "null") {
            $(".titletable").children("div").each(function() { 
                $(this).html("");
            });
            $(".title .condition").text("ただいま予報が利用できません。");
            return;
        }

        rainRadar.buildView();
		if(rainRadar.currentStatus === null) rainRadar.resize();
		
		forecast.populateForecast();
    };

    this.buildView = function() {
        rainRadar.setSummaryView();

		locationManager.setBookmarkStatus({
			lat: userLat,
			long: userLong,
			name: userLocation
		});

        rainRadar.setOutlookView();
		$(".forecast .view-title").text(userLocation);
    };

    this.setSummaryView = function() {
		var temperatureMax = twc.today('temperatureMax');
		temperatureMax = (temperatureMax != "-") ? temperatureMax + '&deg; ' : '-&deg; ';

        $(".title .name").text(userLocation);
		$(".title .tempicon").html('<i class="wi wi-' + twc.today_DayPart('_wi_icon') + '"></i>');
		$(".title .tempdeg").html(' ' + twc.v2obs('temp') + '&deg;');
		$(".title .condition").html(twc.today_DayPart('wxPhraseLong'));

		$(".title .rangetemp").html('<i class="wi wi-wind from-' + twc.v2obs('wdir') + '-deg windicon"></i><span class="text">' + twc.v2obs('wdir_cardinal') + ' ' + twc.v2obs('wspd') + ' m/s</span>');
		
		$(".title .today").html(temperatureMax + twc.today('temperatureMin') + '&deg;&nbsp;&middot;&nbsp;湿度 ' + twc.v2obs('rh') + '%');
		
		rainRadar.currentStatus = twc.today_DayPart('_wi_icon');
	};

    this.setOutlookView = function() {
		var hr_time = null; var hr_time_f = null; var outlook_block = null;

        $("#scroller ul").html("");
		$("#outlook").css("background-color", "rgba(0, 0, 0, 0.05)");

		for(var i = 0; i < 24; i++) {
			hr_time = moment(twc.hourly('validTimeLocal', i));
			hr_time_f = hr_time.format("ha");
			outlook_block = $('<li class="item"><div class="time">' + hr_time_f + '</div><i class="wi wi-' + twc.hourly('_wi_icon', i) + '"></i> ' + twc.hourly('temperature', i) + '&deg;</li>');
			$("#scroller ul").append(outlook_block);
		}

		var scrollerWidth = 0;
		$.each($("#scroller ul li"), function(index, elem) {
			scrollerWidth += parseInt($(elem).width());
		});

		$("#scroller").css("width", scrollerWidth + 20);

		if(rainRadar.outlookScroll === null) {
			rainRadar.outlookScroll = new IScroll('#outlook', {
				scrollX: true,
				scrollY: false,
				bounce: false
			});
		}
		else {
			rainRadar.outlookScroll.refresh();
		}
    };

	// ==========================
    // Loops and Event Functions 
    // ==========================
	this.setIntervalRefreshData = function() {
		clearTimers();

		intervalRefreshData = setInterval(function() {
			if(!($("#radar-container").hasClass("zoom"))) {
				adsManager.runAds();
				rainRadar.getUserLocation();
			}
		}, config.timer.refreshData);
    };
	
	this.resize = function() {
		if(rainRadar.boolMapLoaded === true) rainRadar.resizer.resizeView();
    };

	this.showPopUp = function() {
		$(".popup").fadeIn('fast');
	};
	
	this.hidePopUp = function() {
		$(".popup").fadeOut('fast');
	};
}
